/******************************************************************

	MUSCLE SmartCard Development ( http://www.linuxnet.com )
	    Title  : dyn_unix.c
	    Package: pcsc lite
            Author : David Corcoran
            Date   : 8/12/99
	    License: Copyright (C) 1999 David Corcoran
	             <corcoran@linuxnet.com>
            Purpose: This abstracts dynamic library loading 
                     functions and timing. 

********************************************************************/

#include <wintypes.h>
#include <pcsclite.h>
#include <dyn_generic.h>
#include <debuglog.h>
#include <string.h>
#include <dlfcn.h>

int DYN_LoadLibrary( void** pvLHandle, char* pcLibrary ) {

  char *error;

  /* Zero out everything */
  error=0;

  *pvLHandle = 0;
  *pvLHandle = dlopen( pcLibrary, RTLD_LAZY );

  if ( (error = dlerror()) != 0 ) {
    DebugLogC("DYN_LoadLibrary: dlerror() reports %s\n", error, __FILE__,
               __LINE__ );
    return SCARD_F_UNKNOWN_ERROR;
  }

  return SCARD_S_SUCCESS;
}

int DYN_CloseLibrary( void** pvLHandle ) {

  char *error;

  /* Zero out everything */
  error=0;

  dlclose( *pvLHandle );
  *pvLHandle = 0;

  if ( (error = dlerror()) != 0 ) {
    DebugLogC("DYN_CloseLibrary: dlerror() reports %s\n", error, __FILE__,
               __LINE__ );
    return SCARD_F_UNKNOWN_ERROR;
  }

  return SCARD_S_SUCCESS;
}

int DYN_GetAddress( void* pvLHandle, void** pvFHandle, char* pcFunction ) {

  int rv;
  char *error;
  char *pcFunctionName;

  /* Zero out everything */
  rv=0; error=0; pcFunctionName=0;

  pcFunctionName = pcFunction;

  *pvFHandle = 0;
  *pvFHandle = dlsym( pvLHandle, pcFunctionName );

  if ( (error = dlerror()) != 0 ) {
    DebugLogC("DYN_GetAddress: dlerror() reports %s\n", error, __FILE__,
               __LINE__ );
    rv = SCARD_F_UNKNOWN_ERROR;
  } else {
    rv = SCARD_S_SUCCESS;
  }

  return rv;
}

